@@ -106,7 +106,8 @@ typedef enum rnd_hid_attr_ev_e {
 } rnd_hid_attr_ev_t;
 
 typedef enum rnd_hid_fsd_flags_e { /* bitwise OR, used by file selection dialog (fsd) */
-	RND_HID_FSD_READ = 1             /* when set let the user pick existing files only ("load"); if unset, both esiting and non-existing files are accepted */
+	RND_HID_FSD_READ = 1,            /* when set let the user pick existing files only ("load"); if unset, both esiting and non-existing files are accepted */
+	RND_HID_FSD_MULTI = 32           /* [4.1.0] let the user select multiple files - when RND_HID_FSD_READ is also specified*/
 } rnd_hid_fsd_flags_t;
 
 typedef struct {
@@ -535,8 +536,11 @@ struct rnd_hid_s {
 	/*** (these should be upper, but the struct has to be extended at spares
 	     for binary compatibility) ***/
 
+	/* [4.1.0] Move focus to the widget addressed by idx. Returns 0 on success. */
+	int (*attr_dlg_widget_focus)(void *hid_ctx, int idx);
+
 	/* Spare: see doc/developer/spare.txt */
-	void (*spare_f1)(void), (*spare_f2)(void), (*spare_f3)(void), (*spare_f4)(void), (*spare_f5)(void), (*spare_f6)(void);
+	void (*spare_f2)(void), (*spare_f3)(void), (*spare_f4)(void), (*spare_f5)(void), (*spare_f6)(void);
 	long spare_l1, spare_l2, spare_l3, spare_l4, spare_l5, spare_l6, spare_l7, spare_l8;
 	void *spare_p1, *spare_p2, *spare_p3, *spare_p4, *spare_p5, *spare_p6;
 	double spare_d1, spare_d2, spare_d3, spare_d4, spare_d5, spare_d6;
@@ -556,7 +560,8 @@ struct rnd_hid_expose_ctx_s {
 	long spare_l1, spare_l2, spare_l3, spare_l4;
 	void *spare_p1, *spare_p2, *spare_p3, *spare_p4;
 	double spare_d1, spare_d2, spare_d3, spare_d4;
-	rnd_coord_t spare_c1, spare_c2, spare_c3, spare_c4;
+	rnd_coord_t coord_per_pix;       /* [4.1.0] current zoom level */
+	rnd_coord_t spare_c2, spare_c3, spare_c4;
 };
 
 typedef void (*rnd_hid_expose_t)(rnd_hid_t *hid, const rnd_hid_expose_ctx_t *ctx);
