@@ -5,7 +5,7 @@
  *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996 Thomas Nau
  *  Copyright (C) 2004 harry eaton
- *  Copyright (C) 2016..2021 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2016..2021,2024 Tibor 'Igor2' Palinkas
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -34,10 +34,6 @@
 #include <locale.h>
 #include <ctype.h>
 
-#ifdef __WIN32__
-#include <wchar.h>
-#endif
-
 #include <genvector/vts0.h>
 
 #include <puplug/os_dep_fs.h>
@@ -56,17 +52,16 @@
 #include <librnd/hid/hid_dad_unit.h>
 #include <librnd/core/misc_util.h>
 #include <librnd/core/compat_misc.h>
-#include <librnd/core/compat_inc.h>
 #include <librnd/core/compat_fs.h>
 #include <librnd/core/file_loaded.h>
 #include <librnd/core/hidlib.h>
 #include <librnd/core/rnd_conf.h>
 #include <librnd/core/conf.h>
+#include <librnd/core/paths.h>
 #include <librnd/core/project.h>
 #include <librnd/hid/grid.h>
 #include <librnd/core/funchash.h>
 #include <librnd/hid/hid_menu.h>
-#include <librnd/core/compat_lrealpath.h>
 #include <librnd/core/safe_fs.h>
 #include <librnd/hid/buildin.hidlib.h>
 #include <librnd/hid/tool.h>
@@ -104,12 +99,23 @@ void rnd_hid_init()
 	rnd_render = rnd_gui = rnd_hid_nogui_get_hid();
 
 #ifdef LIBRNDLIBDIR
+#ifdef RND_WANT_FLOATING_FHS
+	tmp = rnd_concat(rnd_w32_root, RND_DIR_SEPARATOR_S, LIBRNDLIBDIR_RELATIVE, RND_DIR_SEPARATOR_S, "plugins", RND_DIR_SEPARATOR_S, HOST, NULL);
+	rnd_plugin_add_dir(tmp);
+	free(tmp);
+
+	tmp = rnd_concat(rnd_w32_root, RND_DIR_SEPARATOR_S, LIBRNDLIBDIR_RELATIVE, RND_DIR_SEPARATOR_S, "plugins", NULL);
+	rnd_plugin_add_dir(tmp);
+	free(tmp);
+
+#else
 	/* librnd's own */
 	tmp = LIBRNDLIBDIR RND_DIR_SEPARATOR_S "plugins" RND_DIR_SEPARATOR_S HOST;
 	rnd_plugin_add_dir(tmp);
 
 	tmp = LIBRNDLIBDIR RND_DIR_SEPARATOR_S "plugins";
 	rnd_plugin_add_dir(tmp);
+#endif
 #endif
 
 	/* host app's */
@@ -496,98 +502,6 @@ const char *rnd_hid_export_fn(const char *filename)
 extern void rnd_hid_dlg_uninit(void);
 extern void rnd_hid_dlg_init(void);
 
-char *rnd_exec_prefix(char *argv0, const char *bin_dir, const char *bin_dir_to_execprefix)
-{
-	size_t l;
-	int haspath;
-	char *t1, *t2;
-	int found_bindir = 0;
-	char *exec_prefix = NULL;
-	char *bindir = NULL;
-
-
-	/* see if argv0 has enough of a path to let lrealpath give the
-	   real path.  This should be the case if you invoke pcb with
-	   something like /usr/local/bin/pcb or ./pcb or ./foo/pcb
-	   but if you just use pcb and it exists in your path, you'll
-	   just get back pcb again. */
-	haspath = (strchr(argv0, RND_DIR_SEPARATOR_C) != NULL);
-
-#ifdef DEBUG
-	printf("rnd_exec_prefix (%s): haspath = %d\n", argv0, haspath);
-#endif
-
-	if (haspath) {
-		bindir = rnd_lrealpath(argv0);
-		if (bindir == NULL)
-			bindir = rnd_strdup(argv0);
-		found_bindir = 1;
-	}
-	else {
-		char *path, *p, *tmps;
-		struct stat sb;
-		int r;
-
-		tmps = getenv("PATH");
-
-		if (tmps != NULL) {
-			path = rnd_strdup(tmps);
-
-			/* search through the font path for a font file */
-			for (p = strtok(path, RND_PATH_DELIMETER); p && *p; p = strtok(NULL, RND_PATH_DELIMETER)) {
-#ifdef DEBUG
-				printf("Looking for %s in %s\n", argv0, p);
-#endif
-				if ((tmps = (char *) malloc((strlen(argv0) + strlen(p) + 2) * sizeof(char))) == NULL) {
-					fprintf(stderr, "rnd_exec_prefix():  malloc failed\n");
-					exit(1);
-				}
-				sprintf(tmps, "%s%s%s", p, RND_DIR_SEPARATOR_S, argv0);
-				r = stat(tmps, &sb);
-				if (r == 0) {
-#ifdef DEBUG
-					printf("Found it:  \"%s\"\n", tmps);
-#endif
-					bindir = rnd_lrealpath(tmps);
-					if (bindir == NULL)
-						bindir = rnd_strdup(tmps);
-					found_bindir = 1;
-					free(tmps);
-					break;
-				}
-				free(tmps);
-			}
-			free(path);
-		}
-	}
-
-	if (found_bindir) {
-		/* strip off the executable name leaving only the path */
-		t2 = NULL;
-		t1 = strchr(bindir, RND_DIR_SEPARATOR_C);
-		while (t1 != NULL && *t1 != '\0') {
-			t2 = t1;
-			t1 = strchr(t2 + 1, RND_DIR_SEPARATOR_C);
-		}
-		if (t2 != NULL)
-			*t2 = '\0';
-	}
-	else {
-		/* we have failed to find out anything from argv[0] so fall back to the original install prefix */
-		bindir = rnd_strdup(bin_dir);
-	}
-
-	/* now find the path to exec_prefix */
-	l = strlen(bindir) + 1 + strlen(bin_dir_to_execprefix) + 1;
-	if ((exec_prefix = (char *) malloc(l * sizeof(char))) == NULL) {
-		fprintf(stderr, "rnd_exec_prefix():  malloc failed\n");
-		exit(1);
-	}
-	sprintf(exec_prefix, "%s%s%s", bindir, RND_DIR_SEPARATOR_S, bin_dir_to_execprefix);
-	free(bindir);
-	return exec_prefix;
-}
-
 static void hidlib_gui_init_ev(rnd_design_t *hidlib, void *user_data, int argc, rnd_event_arg_t argv[])
 {
 	rnd_tool_gui_init();
@@ -830,26 +744,6 @@ int rnd_gui_parse_arguments(int autopick_gui, int *hid_argc, char **hid_argv[])
 	return 0;
 }
 
-#ifdef __WIN32__
-/* truncate the last dir segment; returns remaining length or 0 on failure */
-static int truncdir(char *dir)
-{
-	char *s;
-
-	for(s = dir + strlen(dir) - 1; s >= dir; s--) {
-		if ((*s == '/') || (*s == '\\')) {
-			*s = '\0';
-			return s - dir;
-		}
-	}
-	*dir = '\0';
-	return 0;
-}
-extern int rnd_mkdir_(const char *path, int mode);
-char *rnd_w32_root;
-char *rnd_w32_libdir, *rnd_w32_bindir, *rnd_w32_sharedir, *rnd_w32_cachedir;
-#endif
-
 void rnd_fix_locale_and_env_()
 {
 	static const char *lcs[] = { "LANG", "LC_NUMERIC", "LC_ALL", NULL };
@@ -862,40 +756,6 @@ void rnd_fix_locale_and_env_()
 		rnd_setenv(*lc, "C", 1);
 
 	setlocale(LC_ALL, "C");
-
-
-#ifdef __WIN32__
-	{
-		char *s, exedir[MAX_PATH];
-		wchar_t *w, wexedir[MAX_PATH];
-
-		if (!GetModuleFileNameW(NULL, wexedir, MAX_PATH)) {
-			fprintf(stderr, "%s: GetModuleFileNameW(): failed to determine executable full path\n", rnd_app.package);
-			exit(1);
-		}
-
-		for(w = wexedir, s = exedir; *w != 0; w++)
-			s += wctomb(s, *w);
-		*s = '\0';
-
-		truncdir(exedir);
-
-		for(s = exedir; *s != '\0'; s++)
-			if (*s == '\\')
-				*s = '/';
-
-		rnd_w32_bindir = rnd_strdup(exedir);
-		truncdir(exedir);
-		rnd_w32_root = rnd_strdup(exedir);
-		rnd_w32_libdir = rnd_concat(exedir, "/lib/pcb-rnd", NULL);
-		rnd_w32_sharedir = rnd_concat(exedir, "/share/pcb-rnd", NULL);
-
-		rnd_w32_cachedir = rnd_concat(rnd_w32_root, "/cache", NULL);
-		rnd_mkdir_(rnd_w32_cachedir, 0755);
-
-/*		printf("WIN32 root='%s' libdir='%s' sharedir='%s'\n", rnd_w32_root, rnd_w32_libdir, rnd_w32_sharedir);*/
-	}
-#endif
 }
 
 static int rnd_pcbhl_main_arg_match(const char *in, const char *shrt, const char *lng)
