@@ -159,6 +159,7 @@ static int pa_coll_jump(rnd_vnode_t **cur, pa_direction_t *cdir, pa_jump_rule_t
 		if (!e->flg.mark && rule(d->poly, e, &newdir)) {
 			if (((d->side == 'N') && (newdir == PA_FORWARD)) || ((d->side == 'P') && (newdir == PA_BACKWARD))) {
 				rnd_vnode_t *nnd = (newdir == PA_FORWARD) ? e->next : e;
+				(void)nnd; /* suppress warning on unused */
 				DEBUG_JUMP_COORDS("jump leaving node", nnd, "");
 
 				*cur = d->parent;
@@ -692,7 +693,7 @@ RND_INLINE void pa_polyarea_collect_separated(jmp_buf *e, rnd_pline_t *A, rnd_po
 	   a hole; this hole did not get labelled but if it intersects b, it will
 	   still change the final output. Test case: bdale1 and bdale2 */
 	{
-		rnd_pline_t *pl, **next;
+		rnd_pline_t *pl, *next;
 		for(pl = *holes; pl != NULL; pl = next) {
 			next = pl->next;
 			if ((pl->flg.orphaned) && (pl->flg.llabel == PA_PLL_ISECTED) && (pl->head->flg.plabel == PA_PTL_UNKNWN)) {
