@@ -2,9 +2,9 @@
  *                            COPYRIGHT
  *
  *  libpolybool, 2D polygon bool operations
- *  Copyright (C) 2023 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2023, 2024 Tibor 'Igor2' Palinkas
  *
- *  (Supported by NLnet NGI0 Entrust in 2023)
+ *  (Supported by NLnet NGI0 Entrust in 2023,2024)
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -25,14 +25,6 @@
  *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
- *  This is a full rewrite of pcb-rnd's (and PCB's) polygon lib originally
- *  written by Harry Eaton in 2006, in turn building on "poly_Boolean: a
- *  polygon clip library" by Alexey Nikitin, Michael Leonov from 1997 and
- *  "nclip: a polygon clip library" Klamer Schutte from 1993.
- *
- *  English translation of the original paper the lib is largely based on:
- *  https://web.archive.org/web/20160418014630/http://www.complex-a5.ru/polyboolean/downloads/polybool_eng.pdf
- *
  */
 
 /* Check validity/integrity of polyareas and polylines */
@@ -109,12 +101,12 @@ do { \
 #endif
 
 
-RND_INLINE rnd_bool PA_CHK_ERROR(pa_chk_res_t *res, const char *fmt, ...)
+RND_INLINE rnd_bool PA_CHK_ERROR(pa_chk_res_t *res, const char *first, ...)
 {
 #ifndef NDEBUG
 	va_list ap;
-	va_start(ap, fmt);
-	rnd_vsnprintf(res->msg, sizeof(res->msg), fmt, ap);
+	va_start(ap, first);
+	vsnprints(res->msg, sizeof(res->msg), first, ap);
 	va_end(ap);
 #endif
 	return rnd_true;
@@ -137,16 +129,16 @@ rnd_vnode_t *pa_check_find_close_node(rnd_vector_t intersection, rnd_vnode_t *pn
 	d = rnd_vect_dist2(intersection, pn->point);
 	dn = rnd_vect_dist2(intersection, pn->next->point);
 
-	if ((d < RND_POLY_ENDP_EPSILON) && (dn < RND_POLY_ENDP_EPSILON)) {
+	if ((d < RND_POLY_VALID_ENDP_EPSILON) && (dn < RND_POLY_VALID_ENDP_EPSILON)) {
 		/* line segment too short both ends are close - return the closer one */
 		if (d < dn)
 			return pn;
 		return pn->next;
 	}
 
-	if (d < RND_POLY_ENDP_EPSILON)
+	if (d < RND_POLY_VALID_ENDP_EPSILON)
 		return pn;
-	if (dn < RND_POLY_ENDP_EPSILON)
+	if (dn < RND_POLY_VALID_ENDP_EPSILON)
 		return pn->next;
 	return NULL;
 }
@@ -241,7 +233,7 @@ RND_INLINE rnd_bool pa_pline_check_(rnd_pline_t *a, pa_chk_res_t *res)
 	assert(a != NULL);
 
 	if (a->Count < 3)
-		return PA_CHK_ERROR(res, "pline with points too few at %ld;%ld", a->head->point[0], a->head->point[1]);
+		return PA_CHK_ERROR(res, "pline with points too few at ", Pvnodep(a->head));
 
 	a1 = a->head;
 	do {
@@ -251,14 +243,14 @@ RND_INLINE rnd_bool pa_pline_check_(rnd_pline_t *a, pa_chk_res_t *res)
 		if ((a1->prev->point[0] == a1->next->point[0]) && (a1->prev->point[1] == a1->next->point[1])) {
 			PA_CHK_LINE(a1->point[0], a1->point[1], a1->next->point[0], a1->next->point[1]);
 			PA_CHK_LINE(a1->point[0], a1->point[1], a1->prev->point[0], a1->prev->point[1]);
-			return PA_CHK_ERROR(res, "lines overlap at stub %mm;%mm (full)", a1->point[0], a1->point[1]);
+			return PA_CHK_ERROR(res, "lines overlap at stub ", Pvnodep(a1), " (full)", 0);
 		}
 
 		/* More expensive test: partly overlapping adjacent lines */
 		if (pa_is_node_on_line(a1->next, a1, a1->prev) || pa_is_node_on_line(a1->prev, a1, a1->next)) {
 			PA_CHK_LINE(a1->point[0], a1->point[1], a1->next->point[0], a1->next->point[1]);
 			PA_CHK_LINE(a1->point[0], a1->point[1], a1->prev->point[0], a1->prev->point[1]);
-			return PA_CHK_ERROR(res, "lines overlap at stub %mm;%mm (partial)", a1->point[0], a1->point[1]);
+			return PA_CHK_ERROR(res, "lines overlap at stub ", Pvnodep(a1), " (partial)", 0);
 		}
 
 		do {
@@ -273,7 +265,7 @@ RND_INLINE rnd_bool pa_pline_check_(rnd_pline_t *a, pa_chk_res_t *res)
 			if (icnt > 1) { 
 				PA_CHK_MARK(a1->point[0], a1->point[1]);
 				PA_CHK_MARK(a2->point[0], a2->point[1]);
-				return PA_CHK_ERROR(res, "icnt > 1 (%d) at %ld;%ld or  %ld;%ld", icnt, a1->point[0], a1->point[1], a2->point[0], a2->point[1]);
+				return PA_CHK_ERROR(res, "icnt > 1 (", Pint(icnt) ,") at ", Pnodep(a1), " or ",  Pnodep(a2), 0);
 			}
 #endif
 
@@ -286,7 +278,7 @@ RND_INLINE rnd_bool pa_pline_check_(rnd_pline_t *a, pa_chk_res_t *res)
 				/* intersection in the middle of two lines */
 				PA_CHK_LINE(a1->point[0], a1->point[1], a1->next->point[0], a1->next->point[1]);
 				PA_CHK_LINE(a2->point[0], a2->point[1], a2->next->point[0], a2->next->point[1]);
-				return PA_CHK_ERROR(res, "lines cross between %mm;%mm and %mm;%mm", a1->point[0], a1->point[1], a2->point[0], a2->point[1]);
+				return PA_CHK_ERROR(res, "lines cross between ", Pvnodep(a1), " and ", Pvnodep(a2), 0);
 			}
 			else if (hit1 == NULL) {
 				/* An end-point of a2 touched somewhere along the length of a1. Check
@@ -296,7 +288,7 @@ RND_INLINE rnd_bool pa_pline_check_(rnd_pline_t *a, pa_chk_res_t *res)
 				if ((pa_vect_inside_sect(hit2, a1->point) != pa_vect_inside_sect(hit2, a1->next->point)) && !pa_chk_ll_olap(hit2, a2, a1)) {
 					PA_CHK_MARK(a1->point[0], a1->point[1]);
 					PA_CHK_MARK(hit2->point[0], hit2->point[1]);
-					return PA_CHK_ERROR(res, "plines crossing (1) at %mm;%mm (%ld;%ld)", a1->point[0], a1->point[1], a1->point[0], a1->point[1]);
+					return PA_CHK_ERROR(res, "plines crossing (1) at ", Pvnodep(a1), 0);
 				}
 			}
 			else if (hit2 == NULL) {
@@ -307,7 +299,7 @@ RND_INLINE rnd_bool pa_pline_check_(rnd_pline_t *a, pa_chk_res_t *res)
 				if ((pa_vect_inside_sect(hit1, a2->point) != pa_vect_inside_sect(hit1, a2->next->point)) && !pa_chk_ll_olap(hit1, a1, a2)) {
 					PA_CHK_MARK(a2->point[0], a2->point[1]);
 					PA_CHK_MARK(hit1->point[0], hit1->point[1]);
-					return PA_CHK_ERROR(res, "plines crossing (2) at %mm;%mm (%ld;%ld)", a2->point[0], a2->point[1], a2->point[0], a2->point[1]);
+					return PA_CHK_ERROR(res, "plines crossing (2) at ", Pvnodep(a2), 0);
 				}
 			}
 			else {
@@ -329,7 +321,7 @@ RND_INLINE rnd_bool pa_pline_check_(rnd_pline_t *a, pa_chk_res_t *res)
 					if ((polarity[1] != polarity[0]) || (polarity[2] != polarity[0]) || (polarity[3] != polarity[0])) {
 						PA_CHK_MARK(hit1->point[0], hit2->point[1]);
 						PA_CHK_MARK(hit2->point[0], hit2->point[1]);
-						return PA_CHK_ERROR(res, "plines crossing/twisting (3) at %mm;%mm (%ld;%ld) or %mm;%mm (%ld;%ld)", hit1->point[0], hit1->point[1], hit1->point[0], hit1->point[1], hit2->point[0], hit2->point[1], hit2->point[0], hit2->point[1]);
+						return PA_CHK_ERROR(res, "plines crossing/twisting (3) at ", Pvnodep(hit1), " or ", Pvnodep(hit2), 0);
 					}
 				}
 			}
@@ -357,8 +349,8 @@ static void rnd_poly_valid_report(rnd_pline_t *c, rnd_vnode_t *pl, pa_chk_res_t
 	if (val < min) min = val; \
 	if (val > max) max = val;
 	if (chk != NULL)
-		rnd_fprintf(stderr, "Details: %s\n", chk->msg);
-	rnd_fprintf(stderr, "!!!animator start\n");
+		fprintf(stderr, "Details: %s\n", chk->msg);
+	fprintf(stderr, "!!!animator start\n");
 	v = pl;
 	do {
 		n = v->next;
@@ -370,36 +362,53 @@ static void rnd_poly_valid_report(rnd_pline_t *c, rnd_vnode_t *pl, pa_chk_res_t
 
 	small = is_small(minx) && is_small(miny) && is_small(maxx) && is_small(maxy);
 
-	rnd_fprintf(stderr, "scale 1 -1\n");
-	if (small)
-		rnd_fprintf(stderr, "viewport %ld %ld - %ld %ld\n", (long)minx, (long)miny, (long)maxx, (long)maxy);
-	else
+	fprintf(stderr, "scale 1 -1\n");
+
+#ifdef RND_API_VER
+	if (!small)
 		rnd_fprintf(stderr, "viewport %mm %mm - %mm %mm\n", minx, miny, maxx, maxy);
-	rnd_fprintf(stderr, "frame\n");
+	else
+#endif
+		fprintf(stderr, "viewport %ld %ld - %ld %ld\n", (long)minx, (long)miny, (long)maxx, (long)maxy);
+
+	fprintf(stderr, "frame\n");
 	v = pl;
 	do {
 		n = v->next;
-		if (small)
-			rnd_fprintf(stderr, "line %ld %ld %ld %ld\n", (long)v->point[0], (long)v->point[1], (long)n->point[0], (long)n->point[1]);
-		else
+#ifdef RND_API_VER
+		if (!small)
 			rnd_fprintf(stderr, "line %#mm %#mm %#mm %#mm\n", v->point[0], v->point[1], n->point[0], n->point[1]);
+		else
+#endif
+			fprintf(stderr, "line %ld %ld %ld %ld\n", (long)v->point[0], (long)v->point[1], (long)n->point[0], (long)n->point[1]);
 	}
 	while ((v = v->next) != pl);
 
 	if ((chk != NULL) && (chk->marks > 0)) {
-		int n, MR = small ? 100 : RND_MM_TO_COORD(0.05);
+		int n, MR;
+
+
+#ifdef RND_API_VER
+		MR = small ? 100 : RND_MM_TO_COORD(0.05);
+#else
+		MR = small ? 100 : 1000;
+#endif
+
 		fprintf(stderr, "color #770000\n");
 		for(n = 0; n < chk->marks; n++) {
-			if (small) {
-				rnd_fprintf(stderr, "! X at %ld %ld\n", (long)chk->x[n], (long)chk->y[n]);
-				rnd_fprintf(stderr, "line %ld %ld %ld %ld\n", (long)chk->x[n]-MR, (long)chk->y[n]-MR, (long)chk->x[n]+MR, (long)chk->y[n]+MR);
-				rnd_fprintf(stderr, "line %ld %ld %ld %ld\n", (long)chk->x[n]-MR, (long)chk->y[n]+MR, (long)chk->x[n]+MR, (long)chk->y[n]-MR);
-			}
-			else {
+#ifdef RND_API_VER
+			if (!small) {
 				rnd_fprintf(stderr, "! X at %#mm %#mm\n", (long)chk->x[n], (long)chk->y[n]);
 				rnd_fprintf(stderr, "line %#mm %#mm %#mm %#mm\n", chk->x[n]-MR, chk->y[n]-MR, chk->x[n]+MR, chk->y[n]+MR);
 				rnd_fprintf(stderr, "line %#mm %#mm %#mm %#mm\n", chk->x[n]-MR, chk->y[n]+MR, chk->x[n]+MR, chk->y[n]-MR);
 			}
+			else
+#endif
+			{
+				fprintf(stderr, "! X at %ld %ld\n", (long)chk->x[n], (long)chk->y[n]);
+				fprintf(stderr, "line %ld %ld %ld %ld\n", (long)chk->x[n]-MR, (long)chk->y[n]-MR, (long)chk->x[n]+MR, (long)chk->y[n]+MR);
+				fprintf(stderr, "line %ld %ld %ld %ld\n", (long)chk->x[n]-MR, (long)chk->y[n]+MR, (long)chk->x[n]+MR, (long)chk->y[n]-MR);
+			}
 		}
 	}
 
@@ -407,10 +416,12 @@ static void rnd_poly_valid_report(rnd_pline_t *c, rnd_vnode_t *pl, pa_chk_res_t
 		int n;
 		fprintf(stderr, "color #990000\n");
 		for(n = 0; n < chk->lines; n++)
-			if (small)
-				fprintf(stderr, "line %ld %ld %ld %ld\n", (long)chk->x1[n], (long)chk->y1[n], (long)chk->x2[n], (long)chk->y2[n]);
-			else
+#ifdef RND_API_VER
+			if (!small)
 				rnd_fprintf(stderr, "line %#mm %#mm %#mm %#mm\n", chk->x1[n], chk->y1[n], chk->x2[n], chk->y2[n]);
+			else
+#endif
+				fprintf(stderr, "line %ld %ld %ld %ld\n", (long)chk->x1[n], (long)chk->y1[n], (long)chk->x2[n], (long)chk->y2[n]);
 	}
 
 	fprintf(stderr, "flush\n");
@@ -424,20 +435,22 @@ static void rnd_poly_valid_report(rnd_pline_t *c, rnd_vnode_t *pl, pa_chk_res_t
 
 rnd_bool rnd_poly_valid_island(rnd_polyarea_t *p)
 {
-	rnd_pline_t *n;
+	rnd_pline_t *n, *h;
+	rnd_rtree_it_t it;
+	rnd_rtree_box_t bbox;
 	pa_chk_res_t chk;
 
 	/* Broken cyclic list: if p's prev or next is itself, then the other neighbour
 	   ptr needs to be itself too */
 	if ((p->b == p) && (p->f != p)) {
 #ifndef NDEBUG
-		rnd_fprintf(stderr, "Invalid polyarea ->f\n");
+		fprintf(stderr, "Invalid polyarea ->f\n");
 #endif
 		return rnd_false;
 	}
 	if ((p->f == p) && (p->b != p)) {
 #ifndef NDEBUG
-		rnd_fprintf(stderr, "Invalid polyarea ->f\n");
+		fprintf(stderr, "Invalid polyarea ->f\n");
 #endif
 		return rnd_false;
 	}
@@ -449,7 +462,7 @@ rnd_bool rnd_poly_valid_island(rnd_polyarea_t *p)
 
 	if (p->contours->flg.orient == RND_PLF_INV) {
 #ifndef NDEBUG
-		rnd_fprintf(stderr, "Invalid Outer pline: wrong orientation (shall be positive)\n");
+		fprintf(stderr, "Invalid Outer pline: wrong orientation (shall be positive)\n");
 		rnd_poly_valid_report(p->contours, p->contours->head, NULL);
 #endif
 		return rnd_false;
@@ -457,14 +470,14 @@ rnd_bool rnd_poly_valid_island(rnd_polyarea_t *p)
 
 	if ((p->contours->tree != NULL) && (p->contours->Count != p->contours->tree->size)) {
 #ifndef NDEBUG
-		rnd_fprintf(stderr, "Invalid Outer pline: rtree size mismatch %ld != %ld\n", (long)p->contours->Count, (long)p->contours->tree->size);
+		fprintf(stderr, "Invalid Outer pline: rtree size mismatch %ld != %ld\n", (long)p->contours->Count, (long)p->contours->tree->size);
 #endif
 		return rnd_false;
 	}
 
 	if (pa_pline_check_(p->contours, &chk)) {
 #ifndef NDEBUG
-		rnd_fprintf(stderr, "Invalid Outer pline: self-intersection\n");
+		fprintf(stderr, "Invalid Outer pline: self-intersection\n");
 		rnd_poly_valid_report(p->contours, p->contours->head, &chk);
 #endif
 		return rnd_false;
@@ -474,31 +487,51 @@ rnd_bool rnd_poly_valid_island(rnd_polyarea_t *p)
 	for(n = p->contours->next; n != NULL; n = n->next) {
 		if (n->flg.orient == RND_PLF_DIR) {
 #ifndef NDEBUG
-			rnd_fprintf(stderr, "Invalid Inner (hole): pline orient (shall be negative)\n");
+			fprintf(stderr, "Invalid Inner (hole): pline orient (shall be negative)\n");
 			rnd_poly_valid_report(n, n->head, NULL);
 #endif
 			return rnd_false;
 		}
 		if ((p->contours->tree != NULL) && (p->contours->Count != p->contours->tree->size)) {
 #ifndef NDEBUG
-			rnd_fprintf(stderr, "Invalid Inner (hole): pline rtree size mismatch %ld != %ld\n", (long)p->contours->Count, (long)p->contours->tree->size);
+			fprintf(stderr, "Invalid Inner (hole): pline rtree size mismatch %ld != %ld\n", (long)p->contours->Count, (long)p->contours->tree->size);
 #endif
 			return rnd_false;
 		}
 		if (pa_pline_check_(n, &chk)) {
 #ifndef NDEBUG
-			rnd_fprintf(stderr, "Invalid Inner (hole): self-intersection\n");
+			fprintf(stderr, "Invalid Inner (hole): self-intersection\n");
 			rnd_poly_valid_report(n, n->head, &chk);
 #endif
 			return rnd_false;
 		}
 		if ((p->contours->tree != NULL) && (!pa_pline_inside_pline(p->contours, n))) {
 #ifndef NDEBUG
-			rnd_fprintf(stderr, "Invalid Inner (hole): overlap with outer\n");
+			fprintf(stderr, "Invalid Inner (hole): overlap with outer\n");
 			rnd_poly_valid_report(n, n->head, NULL);
 #endif
 			return rnd_false;
 		}
+
+		/* verify that hole is not intersecting with other holes */
+		bbox.x1 = n->xmin; bbox.y1 = n->ymin;
+		bbox.x2 = n->xmax; bbox.y2 = n->ymax;
+		for(h = rnd_rtree_first(&it, p->contour_tree, &bbox); h != NULL; h = rnd_rtree_next(&it)) {
+			pa_island_isc_t ires;
+
+			if (h >= n) continue; /* check each pair only once; don't check hole against itself */
+			if (h->flg.orient != RND_PLF_INV) continue; /* check hole-hole only */
+
+			ires = pa_pline_pline_isc(h, n, 0);
+			if ((ires == PA_ISLAND_ISC_CROSS) || (ires == PA_ISLAND_ISC_IN)) {
+#ifndef NDEBUG
+				fprintf(stderr, "Invalid Inner (hole): overlaps with another hole\n");
+				rnd_poly_valid_report(n, n->head, NULL);
+#endif
+				return rnd_false;
+			}
+		}
+
 	}
 	return rnd_true;
 }
@@ -522,7 +555,7 @@ rnd_bool rnd_poly_valid(rnd_polyarea_t *pa)
 			while(q != pa) {
 				if (rnd_polyarea_island_isc(p, q)) {
 #ifndef NDEBUG
-					rnd_fprintf(stderr, "island-island intersect\n");
+					fprintf(stderr, "island-island intersect\n");
 #endif
 					return rnd_false;
 				}
