@@ -4,15 +4,9 @@
 #include <assert.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include <setjmp.h>
 #include <string.h>
 
 #include <librnd/rnd_config.h>
-#include <librnd/core/math_helper.h>
-#include <librnd/core/heap.h>
-#include <librnd/core/rnd_printf.h>
-#include <librnd/core/box.h>
-
 
 #include "polyconf.h"
 
@@ -20,6 +14,7 @@
 #include "rtree.h"
 #include "rtree2_compat.h"
 #include "pa.h"
+#include "pa_prints.h"
 
 #include "pa_math.c"
 #include "pa_vect.c"
